local armada = require 'armada'
local ev = require 'ev'
local u = require 'unix'
local cc = require 'connctrl'
local ffi = require 'ffi'
local C = ffi.C

ffi.cdef[[
typedef int ssize_t;
ssize_t read(int fd, void *buf, size_t count);
]]

local cca_handler = {}

function cca_handler.new(devname, proxy, api, firmware, clk_frequency)
    proxy.cca_dev = api:new(devname, firmware, clk_frequency)
    proxy.cmd_responses = { }
    proxy.cmd_actions = { }
    proxy.pending_tasks = {}
    proxy.int_dev = assert(u.open(devname, u.O_RDONLY + u.O_NONBLOCK))
    proxy.int_watcher = ev.IO.new(cca_handler.reader, proxy.int_dev, ev.READ)
    proxy.int_watcher.cca_proxy = proxy
    proxy.int_watcher:start(ev.Loop.default)

--[[
    local function cc_monitor(loop, w, revents)
        cc.check_for_exception(devname)
    end
    proxy.cc_monitor_timer = ev.Timer.new(cc_monitor, 0.1, 0.1)
    proxy.cc_monitor_timer:start(ev.Loop.default)
]]

    -- Associate cca_handler commands
    for k,v in pairs(cca_handler) do
        if k ~= 'new' then proxy[k] = v end
    end

    -- sync it up
    proxy:cmd_block('setup_sync_ts')
    local sync_time = armada.timing:SendSyncPulse()
    local msg = proxy:cmd_block('get_sync_ts')
    local offset = sync_time - msg.ts + 1
    proxy:cmd_block('ts_offset', offset)
end

function cca_handler:halt(dump_trace)
    self.cca_dev:halt(dump_trace)
end

function cca_handler:cmd_cb(cmd, cb, ...)
--    print('cmd_cb', cmd, ...)
    local cmd_id = self.cca_dev[cmd](self.cca_dev, ...)
    self.cmd_actions[cmd_id] = cb
    return cmd_id
end

function cca_handler:cmd_block(cmd, ...)
--    print('cmd_block', cmd, ...)
    local cmd_id = self:cmd_cb(cmd,
        function (msg) self.cmd_responses[msg.cmd_id] = msg end, ...)
    repeat
        ev.Loop.default:loop('once')
    until self.cmd_responses[cmd_id]
    local msg = self.cmd_responses[cmd_id]
    self.cmd_responses[cmd_id] = nil
    return msg
end

function cca_handler:cmd(cmd, ...)
--    print('cmd', cmd, ...)
    self:cmd_cb(cmd, function () end, ...)
end

function cca_handler:cmd_yield(cmd, ...)
--    print('cmd_yield', cmd, ...)
    local my_coro = assert(coroutine.running())
    self:cmd_cb(cmd, function (msg)
                         assert(coroutine.resume(my_coro, msg))
                     end, ...)
    return assert(coroutine.yield())
end

local drain_buf = ffi.new('char[1]')

function cca_handler.reader_old(loop, w, revents)
    local self = w.cca_proxy

    -- Drain the interrupt event indicator.  The read function returns 1 until
    -- the counter in the driver gets to 0, then it returns -1 with errno set
    -- to EAGAIN.
    while true do
        if C.read(self.int_dev, drain_buf, 1) < 0 then break end

        -- Process the message
        local msg = self.cca_dev:poll(self)
        if msg and type(msg) == 'table' then
            assert(self.cmd_actions[msg.cmd_id])(msg)
            self.cmd_actions[msg.cmd_id] = nil
        end
    end

    if self.launch_coroutine then self:launch_coroutine() end
end

function cca_handler.reader(loop, w, revents)
    local self = w.cca_proxy

    if C.read(self.int_dev, drain_buf, 1) < 0 then return end

    -- Drain the interrupt event indicator.  The read function returns 1 until
    -- the counter in the driver gets to 0, then it returns -1 with errno set
    -- to EAGAIN.
    while true do
        -- Process the message
        local msg = self.cca_dev:poll(self)
        if not msg then break end
        if type(msg) == 'table' then
            assert(self.cmd_actions[msg.cmd_id])(msg)
            self.cmd_actions[msg.cmd_id] = nil
        end
    end

    if self.launch_coroutine then self:launch_coroutine() end
end

function cca_handler:launch_coroutine(func)
    if func then
        table.insert(self.pending_tasks, coroutine.create(func))
    end
    while not self.active_task or coroutine.status(self.active_task) == 'dead'
    do
        self.active_task = table.remove(self.pending_tasks, 1)
        if not self.active_task then break end
        coroutine.resume(self.active_task)
    end
end

function cca_handler:message(msg)
    return false
end

return cca_handler
