local ffi = require 'ffi'
local lj_include = require 'lj_include'

lj_include('linux_types')

ffi.cdef[[
struct pt_regs {
    unsigned long gpr[32];
    unsigned long nip;
    unsigned long msr;
    unsigned long orig_gpr3;
    unsigned long ctr;
    unsigned long link;
    unsigned long xer;
    unsigned long ccr;
    unsigned long mq;
    unsigned long trap;
    unsigned long dar;
    unsigned long dsisr;
    unsigned long result;
};
struct ppc_debug_info {
    __u32 version;
    __u32 num_instruction_bps;
    __u32 num_data_bps;
    __u32 num_condition_regs;
    __u32 data_bp_alignment;
    __u32 sizeof_condition;
    __u64 features;
};
struct ppc_hw_breakpoint {
    __u32 version;
    __u32 trigger_type;
    __u32 addr_mode;
    __u32 condition_mode;
    __u64 addr;
    __u64 addr2;
    __u64 condition_value;
};
]]

local defs = {
    PT_R0 = '0',
    PT_R1 = '1',
    PT_R2 = '2',
    PT_R3 = '3',
    PT_R4 = '4',
    PT_R5 = '5',
    PT_R6 = '6',
    PT_R7 = '7',
    PT_R8 = '8',
    PT_R9 = '9',
    PT_R10 = '10',
    PT_R11 = '11',
    PT_R12 = '12',
    PT_R13 = '13',
    PT_R14 = '14',
    PT_R15 = '15',
    PT_R16 = '16',
    PT_R17 = '17',
    PT_R18 = '18',
    PT_R19 = '19',
    PT_R20 = '20',
    PT_R21 = '21',
    PT_R22 = '22',
    PT_R23 = '23',
    PT_R24 = '24',
    PT_R25 = '25',
    PT_R26 = '26',
    PT_R27 = '27',
    PT_R28 = '28',
    PT_R29 = '29',
    PT_R30 = '30',
    PT_R31 = '31',
    PT_NIP = '32',
    PT_MSR = '33',
    PT_ORIG_R3 = '34',
    PT_CTR = '35',
    PT_LNK = '36',
    PT_XER = '37',
    PT_CCR = '38',
    PT_MQ = '39',
    PT_SOFTE = '39',
    PT_TRAP = '40',
    PT_DAR = '41',
    PT_DSISR = '42',
    PT_RESULT = '43',
    PT_DSCR = '44',
    PT_REGS_COUNT = '44',
    PT_FPR0 = '48',
    PT_VR0 = '82',
    PT_VSR0 = '150',
    PTRACE_GETVRREGS = '0x12',
    PTRACE_SETVRREGS = '0x13',
    PTRACE_GETEVRREGS = '0x14',
    PTRACE_SETEVRREGS = '0x15',
    PTRACE_GETVSRREGS = '0x1b',
    PTRACE_SETVSRREGS = '0x1c',
    PTRACE_GET_DEBUGREG = '0x19',
    PTRACE_SET_DEBUGREG = '0x1a',
    PTRACE_GETREGS = '0xc',
    PTRACE_SETREGS = '0xd',
    PTRACE_GETFPREGS = '0xe',
    PTRACE_SETFPREGS = '0xf',
    PTRACE_GETREGS64 = '0x16',
    PTRACE_SETREGS64 = '0x17',
    PPC_PTRACE_PEEKTEXT_3264 = '0x95',
    PPC_PTRACE_PEEKDATA_3264 = '0x94',
    PPC_PTRACE_POKETEXT_3264 = '0x93',
    PPC_PTRACE_POKEDATA_3264 = '0x92',
    PPC_PTRACE_PEEKUSR_3264 = '0x91',
    PPC_PTRACE_POKEUSR_3264 = '0x90',
    PTRACE_SINGLEBLOCK = '0x100',
    PPC_PTRACE_GETHWDBGINFO = '0x89',
    PPC_PTRACE_SETHWDEBUG = '0x88',
    PPC_PTRACE_DELHWDEBUG = '0x87',
    PPC_DEBUG_FEATURE_INSN_BP_RANGE = '0x0000000000000001',
    PPC_DEBUG_FEATURE_INSN_BP_MASK = '0x0000000000000002',
    PPC_DEBUG_FEATURE_DATA_BP_RANGE = '0x0000000000000004',
    PPC_DEBUG_FEATURE_DATA_BP_MASK = '0x0000000000000008',
    PPC_DEBUG_FEATURE_DATA_BP_DAWR = '0x0000000000000010',
    PPC_BREAKPOINT_TRIGGER_EXECUTE = '0x00000001',
    PPC_BREAKPOINT_TRIGGER_READ = '0x00000002',
    PPC_BREAKPOINT_TRIGGER_WRITE = '0x00000004',
    PPC_BREAKPOINT_MODE_EXACT = '0x00000000',
    PPC_BREAKPOINT_MODE_RANGE_INCLUSIVE = '0x00000001',
    PPC_BREAKPOINT_MODE_RANGE_EXCLUSIVE = '0x00000002',
    PPC_BREAKPOINT_MODE_MASK = '0x00000003',
    PPC_BREAKPOINT_CONDITION_MODE = '0x00000003',
    PPC_BREAKPOINT_CONDITION_NONE = '0x00000000',
    PPC_BREAKPOINT_CONDITION_AND = '0x00000001',
    PPC_BREAKPOINT_CONDITION_EXACT = '0x00000001',
    PPC_BREAKPOINT_CONDITION_OR = '0x00000002',
    PPC_BREAKPOINT_CONDITION_AND_OR = '0x00000003',
    PPC_BREAKPOINT_CONDITION_BE_ALL = '0x00ff0000',
    PPC_BREAKPOINT_CONDITION_BE_SHIFT = '16',
}
local format, cdef, C = string.format, ffi.cdef, ffi.C
for sym, val in pairs(defs) do
    if not pcall(function() return C[sym] end) then
        cdef(format("enum {%s = %s};", sym, val))
    end
end

