--! \file

local common = require 'mx.device_common'
local underscore = require 'underscore'

local m = {}

-- XML used to do initial configuration of channels when proxy starts up
local default_xml = [[<?xml version="1.0" encoding="UTF-8"?>
                <Amplifier>
                    <Connector#conn_num#>
                        <Mode choice="AnalogInChannel">
                            <AnalogInChannel>
                                <Sensor>
                                    <TedsOptions>
                                        <UseTeds>2</UseTeds>
                                    </TedsOptions>
                                </Sensor>
                                <SyncSignal address="1">
                                    <OutputRate>10</OutputRate>
                                </SyncSignal>
                                <OutputUnit>
                                    <UnitId>1100</UnitId>
                                </OutputUnit>
                            </AnalogInChannel>
                        </Mode>
                    </Connector#conn_num#>
                </Amplifier>
]]

-- Function used to return the XML used to set the channel to it's default
-- configuration
function m:get_default_xml(channel_num)
    return default_xml:gsub('#conn_num#', tostring(channel_num))
end

m.json2xpath = {
    transducer_power = '/Amplifier/Connector#channel_num#/Mode[@choice="AnalogInChannel"]/AnalogInChannel/Sensor/Supply',
}
underscore.extend(m.json2xpath, common.json2xpath)

m.physical_unit_map = {
    Voltage = 'V',
    Current = 'mA',
    IEPE = 'C',
}

-- List of parameters in a channel setup that we do NOT want sent off to the MX
-- device
m.ignore_list = {}
underscore.extend(m.ignore_list, common.ignore_list)

-- List of parameters returned by MX device that we don't verify after setting
-- up a channel.
m.dont_verify_list = {}
underscore.extend(m.dont_verify_list, common.dont_verify_list)

-- Specification of the default settings of this channel type
m.default_spec = {
    sample_rate = '10',
    sensor_input = {
        mode = 'volt',
        min = '-10',
        max = '10.000000',
    },
    calibration = {
        define_source_units = {
            purpose = 'define_source_units',
            units = 'V',
        },
        sensor_scaling = {
            purpose = 'sensor_scaling',
            mode = 'defined_slope_intercept',
            slope = '1',
            intercept = '0',
            units = 'V',
        },
        units_conversion = {
            purpose = 'units_conversion',
            mode = 'defined_slope_intercept',
            units = 'V',
            intercept = '0',
            slope = '1',
        },
        rezero = {
            purpose = 'rezero',
            mode = 'defined_slope_intercept',
            intercept = '0.00000',
        },
    },
    transducer_power = '0.000000',
    digital_filter_type = 'none',
    sensor = {
        use_teds = 2,
    },
}

m.phase_delay = {}
m.phase_delay.fixed = 398
m.phase_delay.bessel_iir = {}
m.phase_delay.butterworth_iir = {}
m.phase_delay.linear_phase_fir = {}
m.phase_delay.butterworth_fir = {}

return m
