require('lj_include'){
    'errno',
    'string',
    'sys/types',
    'netinet/in',
}

local ffi = require'ffi'

ffi.cdef[[
enum {
    EVENT_PORT = 319,
    GENERAL_PORT = 320,
};

typedef unsigned char           octet;
typedef unsigned char           enum8_t;

struct uint48 {
    uint16_t    msb;
    uint32_t    lsb;
} __attribute__((packed));

typedef struct uint48   uint48_t;

struct timestamp {
    uint48_t    seconds;
    uint32_t    nanoseconds;
} __attribute__((packed));

typedef struct timestamp        timestamp_t;

typedef octet   clock_id[8];

struct port_identity {
    clock_id    clock_identity;
    uint16_t    port_number;
} __attribute__((packed));

typedef struct port_identity    port_identity_t;

struct msg_hdr {
    uint8_t             transport_specific_message_type;
    uint8_t             reserved_version_ptp;
    uint16_t            message_length;
    uint8_t             domain_number;
    octet               _reserved;
    octet               flag_field[2];
    struct {
        uint48_t        nanoseconds;
        uint16_t        fractional_nanoseconds;
    } correction_field;
    octet               reserved[4];
    port_identity_t     source_port_identity;
    uint16_t            sequence_id;
    uint8_t             control_field;
    int8_t              log_message_interval;
} __attribute__((packed));

typedef struct msg_hdr  msg_hdr_t;

struct sync_msg {
    msg_hdr_t   hdr;
    timestamp_t origin_timestamp;
    uint16_t    ip6_pad;
} __attribute__((packed));

typedef struct sync_msg sync_msg_t;
typedef struct sync_msg delay_req_msg_t;

struct delay_resp_msg {
    msg_hdr_t           hdr;
    timestamp_t         receive_timestamp;
    port_identity_t     requesting_port_identity;
    uint16_t            ip6_pad;
} __attribute__((packed));

typedef struct delay_resp_msg   delay_resp_msg_t;

struct followup_msg {
    msg_hdr_t   hdr;
    timestamp_t precise_origin_timestamp;
    uint16_t    ip6_pad;
} __attribute__((packed));

typedef struct followup_msg     followup_msg_t;

struct clock_quality {
    uint8_t     clock_class;
    enum8_t     clock_accuracy;
    uint16_t    offset_scaled_log_variance;
} __attribute__((packed));

typedef struct clock_quality    clock_quality_t;

struct grandmaster {
    uint8_t             priority1;
    clock_quality_t     clock_quality;
    uint8_t             priority2;
    clock_id            identity;
} __attribute__((packed));

typedef struct grandmaster      grandmaster_t;

struct announce_msg {
    msg_hdr_t           hdr;
    timestamp_t         origin_timestamp;
    int16_t             current_utc_offset;
    uint8_t             reserved;
    grandmaster_t       grandmaster;
    uint16_t            steps_removed;
    enum8_t             time_source;
    uint16_t            ip6_pad;
} __attribute__((packed));

typedef struct announce_msg     announce_msg_t;

typedef struct sync_msg sync_msg_t;
typedef struct sync_msg_ip6     sync_msg_ip6_t;

// Clock state
enum {
    PTP_STATE_INITIALIZING,
    PTP_STATE_FAULTY,
    PTP_STATE_LISTENING,
    PTP_STATE_PASSIVE,
    PTP_STATE_UNCALIBRATED,
    PTP_STATE_SLAVE,
    PTP_STATE_PRE_MASTER,
    PTP_STATE_MASTER,
    PTP_STATE_DISABLED,
};

// Message types
enum {
    PTP_MSG_TYPE_SYNC,
    PTP_MSG_TYPE_DELAY_REQ,
    PTP_MSG_TYPE_PDELAY_REQ,
    PTP_MSG_TYPE_PDELAY_RESP,
    PTP_MSG_TYPE_FOLLOW_UP = 8,
    PTP_MSG_TYPE_DELAY_RESP,
    PTP_MSG_TYPE_PDELAY_RESP_FOLLOW_UP,
    PTP_MSG_TYPE_ANNOUNCE,
    PTP_MSG_TYPE_SIGNALING,
    PTP_MSG_TYPE_MANAGEMENT,
};

// Clock types
enum {
    PTP_CLOCK_TYPE_ATOMIC_CLOCK = 0x10,
    PTP_CLOCK_TYPE_GPS = 0x20,
    PTP_CLOCK_TYPE_TERRESTRIAL_RADIO = 0x30,
    PTP_CLOCK_TYPE_PTP = 0x40,
    PTP_CLOCK_TYPE_NTP = 0x50,
    PTP_CLOCK_TYPE_HAND_SET = 0x60,
    PTP_CLOCK_TYPE_OTHER = 0x90,
    PTP_CLOCK_TYPE_INTERNAL_OSCILLATOR = 0xa0,
};

// PTP version numbers
enum {
    PTP_VERSION_1 = 1,
    PTP_VERSION_2,
};

// PTP Domains
enum {
    PTP_DEFAULT_DOMAIN = 0,
    PTP_ALTERNATE_DOMAIN_1,
    PTP_ALTERNATE_DOMAIN_2,
    PTP_ALTERNATE_DOMAIN_3,
};

// Flags
enum {
    PTP_TWO_STEP_FLAG = 0x2,
};

// Assorted
enum {
    PTP_SLAVE_ONLY = -1,
};

// Delay mechanisms
enum {
    DELAY_MECHANISM_E2E = 1,
    DELAY_MECHANISM_P2P,
    DELAY_DISABLED = 0xfe,
};

// Defaults
enum {
    DEFAULT_MAX_FOREIGN_RECORDS = 5,
    DEFAULT_NUMBER_PORTS = 1,
    DEFAULT_INBOUND_LATENCY_S = 0,
    DEFAULT_INBOUND_LATENCY_NS = 0,
    DEFAULT_OUTBOUND_LATENCY_S = 0,
    DEFAULT_OUTBOUND_LATENCY_NS = 0,
    DEFAULT_DOMAIN_NUMBER = PTP_DEFAULT_DOMAIN,
    DEFAULT_DELAY_MECHANISM = DELAY_MECHANISM_P2P,
    DEFAULT_ANNOUNCE_INTERVAL = 1,
    DEFAULT_UTC_OFFSET = 0,
    DEFAULT_PDELAYREQ_INTERVAL = 1,
    DEFAULT_DELAYREQ_INTERVAL = 3,
    DEFAULT_SYNC_INTERVAL = -3,
    DEFAULT_SYNC_RECEIPT_TIMEOUT = 3,
//    DEFAULT_ANNOUNCE_RECEIPT_TIMEOUT = 6,
    DEFAULT_ANNOUNCE_RECEIPT_TIMEOUT = 3,
    DEFAULT_QUALIFICATION_TIMEOUT = 2,
    DEFAULT_FOREIGN_MASTER_TIME_WINDOW = 4,
    DEFAULT_FOREIGN_MASTER_THRESHOLD = 2,
    DEFAULT_CLOCK_CLASS = 248,
    CLOCK_CLASS_SOMAT_GPS = 216,
    DEFAULT_CLOCK_ACCURACY = 0xfe,
// We do not EVER want QuantumX 'B' devices being a master, this setting will
// prevent that from ever happening.
    DEFAULT_PRIORITY1 = 128,
//    DEFAULT_PRIORITY1 = 127,
    DEFAULT_PRIORITY2 = 128,
    DEFAULT_CLOCK_VARIANCE = -4000,
    DEFAULT_PARENT_STATS = 0,
};

typedef struct foreign_master_record {
    port_identity_t     foreign_master_port_identity;
    uint16_t            foreign_master_announce_messages;
    announce_msg_t      announce;
} foreign_master_record_t;

typedef struct internal_time {
    int32_t     seconds;
    int32_t     nanoseconds;
} internal_time_t;

struct ptp_clock_info {
    octet                       port_uuid_field[6];     // MAC Address of iface
    int                         state;
    char                        ibuf[8192];
    msg_hdr_t                   *msg_hdr;
    announce_msg_t              *announce_msg;
    announce_msg_t              amsg;
    msg_hdr_t                   *msg_hdr_save;
    sync_msg_t                  sync_msg[1];            // Used by one step
    clock_id                    clock_identity;
    int                         two_step_flag;
    int                         number_ports;
    clock_quality_t             clock_quality;
    int                         priority1;
    int                         priority2;
    unsigned int                domain_number;
    int                         slave_only;
    port_identity_t             port_identity;
    port_identity_t             parent_port_identity;
    port_identity_t             old_ppi;
    int                         log_min_delay_req_interval;
    int                         log_announce_interval;
    int                         announce_receipt_timeout;
    int                         log_sync_interval;
    int                         delay_mechanism;
    int                         log_min_pdelay_req_interval;
    unsigned int                version_number;
    int                         R;
    int                         number_foreign_records;
    int                         max_foreign_records;
    foreign_master_record_t     *foreign;

    uint16_t                    sent_sync_sequence_id;
    uint16_t                    recv_sync_sequence_id;
    uint16_t                    sent_announce_sequence_id;
    uint16_t                    sent_delay_req_sequence_id;
    uint16_t                    sent_pdelay_req_sequence_id;
    int                         current_utc_offset;
    int                         waiting_for_follow;
    int                         oo_followup;    // Out-of-order followup ms
    followup_msg_t              oo_fu_msg[1];   // Saved copy of oo followup

    int                         foreign_record_best;
    int                         foreign_record_i;

    grandmaster_t               grandmaster;


    internal_time_t             master_to_slave_delay;
    internal_time_t             slave_to_master_delay;

/*
    internal_time_t             pdelay_req_receive_time;
    internal_time_t             pdelay_req_send_time;
    internal_time_t             pdelay_resp_receive_time;
    internal_time_t             pdelay_resp_send_time;
    internal_time_t             sync_receive_time;
    internal_time_t             delay_req_receive_time;
    internal_time_t             delay_req_send_time;
*/

    int                         time_source;

    internal_time_t             inbound_latency;
    internal_time_t             outbound_latency;

    int                         parent_stats;
    int                         message_activity;

    unsigned char               steps_removed;

    int                         sizeof_sync_msg;
    int                         sizeof_announce_msg;
    int                         sizeof_followup_msg;
    int                         sizeof_delay_req_msg;
    int                         sizeof_delay_resp_msg;
    int                         sizeof_pdelay_req_msg;
    int                         sizeof_pdelay_req_followup_msg;
    int                         sizeof_pdelay_resp_msg;

    int                         transport_is_ipv6;

    int                         current_utc_offset_valid;
    int                         leap59;
    int                         leap61;
    int                         time_traceable;
    int                         frequency_traceable;
    int                         ptp_timescale;

    int                         waiting_delay_req_tx_ts;

    internal_time_t             pot;    // precise origin timestamp
    int                         e2e_mode;

    internal_time_t             delay_ms;
    internal_time_t             delay_sm;

    int                         offset_first_updated;

    int                         num_dropped;
    int                         num_rejected_ts;
    int                         num_double_reject_ts;
    int                         num_rejected_delay_responses;
    int                         num_rejected_delay_response_ts;
    int                         num_adjusted;
    double                      avg_sm_delay;
};

enum {
    ANNOUNCE_RECEIPT_TIMEOUT_TIME = 20,
    ANNOUNCE_INTERVAL_TIME = 16,
};
]]

return ffi
