require 'middleclass'
local gmp = require 'gmp'

local rational = class('rational')
function rational:initialize(val)
    self.q = gmp.new_q()
    if val then
        self:set(val)
    end
end

function rational:set(val)
    local t = type(val)
    if t == 'table' and val.class == rational then
        self.q:set(val.q)
    elseif t == 'number' then
        self.q:set_d(val)
    elseif t == 'string' then
        self:set_string(val)
    end
    return self
end

function rational:mul_exp10(n)
    local mul = gmp.new_z()
    mul:set_ui(10)
    mul:pow_ui(mul, math.abs(n))
    local ref = (n >= 0) and self.q:numref() or self.q:denref()
    ref:mul(ref, mul)
    self.q:canonicalize()
    return self
end

function rational:set_string(str)
    local dot = str:find('%.')
    local e = str:find('[eE]')
    if dot or e then
        local mantissa = str:sub(1, e and (e - 1))
        local exp = e and tonumber(str:sub(e + 1)) or 0
        local num = mantissa
        local dot_exp = 0
        if dot then
            num = mantissa:sub(1, dot - 1) .. mantissa:sub(dot + 1)
            dot_exp = mantissa:len() - dot
        end
        self.q:set_str(num, 10)
        self:mul_exp10(exp - dot_exp)
    else
        self.q:set_str(str, 0)
        self.q:canonicalize()
    end
    return self
end

function rational:tonumber()
    return self.q:get_d()
end

function rational:tonumbers()
    return self.q:get_d(), self.q:numref():get_d(), self.q:denref():get_d()
end

local function force_q(val)
    local t = type(val)
    if t == 'table' and val.class == rational then
        return val.q
    else
        return rational:new(val).q
    end
end

function rational.__add(a, b)
    local r = rational:new()
    r.q:add(force_q(a), force_q(b))
    return r
end

function rational.__sub(a, b)
    local r = rational:new()
    r.q:sub(force_q(a), force_q(b))
    return r
end

function rational.__mul(a, b)
    local r = rational:new()
    r.q:mul(force_q(a), force_q(b))
    return r
end

function rational.__div(a, b)
    local r = rational:new()
    r.q:div(force_q(a), force_q(b))
    return r
end

function rational.__unm(a)
    local r = rational:new()
    r.q:neg(force_q(a))
    return r
end

function rational.__eq(a, b)
    return gmp.q_equal(force_q(a), force_q(b)) ~= 0
end

function rational.__lt(a, b)
    return gmp.q_cmp(force_q(a), force_q(b)) < 0
end

function rational.__le(a, b)
    return gmp.q_cmp(force_q(a), force_q(b)) <= 0
end

function rational.__tostring(a)
    return a.q:get_string()
end

return rational
