local ffi = require 'ffi'
local rational = require 'rational'

require 'strict'

local master_rate = 40000000 -- KLUDGE HARDCODED get from prefs on start!

ffi.cdef[[
typedef double sample_math_type; // int64_t is around 5 times slower on PowerPC

typedef struct sample_clock {
    int64_t current_clock;
    int64_t next_clock;
    sample_math_type numerator;
    sample_math_type denominator;
    sample_math_type base;
    sample_math_type remainder;
    sample_math_type accumulator;
    sample_math_type master_rate;
    sample_math_type half_numerator;
    double float_rate;
} sample_clock;
]]

local c_type = 'sample_clock'

local M = {}

function M:init(--[[rational]] rate, master_clock_rate) 
    local rat = rational:new(rate)
    self.float_rate, self.numerator, self.denominator = rat:tonumbers()
    self.half_numerator = self.numerator / 2
    self.master_rate = master_clock_rate
    self.base = math.floor(
        tonumber(self.master_rate * self.denominator / self.numerator))
    self.remainder = self.master_rate * self.denominator
        - self.base * self.numerator

    self.accumulator = 0
    self.current_clock = 0
    self.next_clock = self.base
end

function M:tick()
    self.current_clock = self.next_clock
    self.accumulator = self.accumulator + self.remainder
    self.next_clock = self.next_clock + self.base
    if self.accumulator > self.half_numerator then
        self.accumulator = self.accumulator - self.numerator
        self.next_clock = self.next_clock + 1
    end
end

return {
    _new = ffi.metatype(c_type, {
        __index = M,
    }),
    new = function(self, --[[rational]] rate, master_rate)
        local c = self._new()
        c:init(rate, master_rate or self.master_rate)
        return c
    end,
    c_type_name = c_type,
    c_type = ffi.typeof(c_type),
    ptr_type = ffi.typeof(c_type .. '*'),
    size = ffi.sizeof(c_type),
    master_rate = master_rate,
    timescale = '1/' .. master_rate,
    secs = function(ts) return tonumber(ts) / tonumber(master_rate) end,
}
