require 'strict'

-- pv_algo.lua - Implements the Peak Valley Processing Algorithm.
-- This may be used by multiple modules.
--

local m = {}

-- Executes the Peak Valley Processing algorithm on data in
-- the given input frame.
-- Parameter:
--     'in_count'  number of input data samples
--     'input_info'  table containing info for one input frame
--     'hysteresis'  change needed before peak/valley determined
--     'pv_array'  array of indices for each peak and valley found
function m.run_pv_algo(in_count, input_info,
                       hysteresis, pv_array)
    local iv = input_info.invals

    for i = 0, in_count - 1 do
        if input_info.state == "initial" then
            if iv[i] < input_info.valley then
                input_info.valley = iv[i]
            elseif iv[i] - input_info.valley > hysteresis then
                -- valley found
                pv_array[#pv_array + 1] = input_info.valley
                input_info.peak = iv[i]
                input_info.state = "peak_search"
            end

            if iv[i] > input_info.peak then
                input_info.peak = iv[i]
            elseif input_info.peak - iv[i] > hysteresis then
                -- peak found
                pv_array[#pv_array + 1] = input_info.peak
                input_info.valley = iv[i]
                input_info.state = "valley_search"
            end
        elseif input_info.state == "valley_search" then
            -- look for a valley
            if iv[i] < input_info.valley then
                input_info.valley = iv[i]
            elseif iv[i] - input_info.valley > hysteresis then
                -- valley found
                pv_array[#pv_array + 1] = input_info.valley
                input_info.peak = iv[i]
                input_info.state = "peak_search"
            end
        elseif input_info.state == "peak_search" then
            -- look for a peak
            if iv[i] > input_info.peak then
                input_info.peak = iv[i]
            elseif input_info.peak - iv[i] > hysteresis then
                -- peak found
                pv_array[#pv_array + 1] = input_info.peak
                input_info.valley = iv[i]
                input_info.state = "valley_search"
            end
        else
            assert(false, 'Invalid PV datamode state for input ' ..
                tostring(input_info.name) .. '(state = ' ..
                tostring(input_info.state) .. ')')
        end
    end
end

return m

