/*jshint white:false */
(function( $, undefined ) {

$.widget( 'ui.somat_reading',
{
    /**
     * @description This attribute defines the natural pixel width of the
     *              gauge. It is used for scale math.
     */
    _normalW     :  100,
    /**
     * @description This attribute defines the natural pixel height of the
     *              gauge. It is used for scale math.
     */
    _normalH     :   35,

    _baseURL     :  $('script:last').prop('src').replace(/[^\/]*$/, ''),

    // private
    'ready'       : false,
    'scale_x'     :   1.0,
    'scale_y'     :   1.0,

    // layout
    'unitX'      :   70,
    'unitY'       :   18,
    'legendX'     :   50,
    'legendY'     :   30,
    'readingX'    :   70,
    'readingY'    :   18,
    'readingFont' :   64,
    'unitFont'    :   40,
    'legendFont'  :   40,

    // dom
    'svg'     :  null,

    /**
     * @description This object stores the instance data of this widget.
     * NOTES:
     *        show_needle : shows a needle instead of a bar when TRUE.
     */
    options: {
        'no_svg'      :      false,
        'skin'        :    'white',
        'top'         :          0,
        'left'        :          0,
        'height'      :         60,
        'width'       :         50,
        'legend'      : 'abcd1234',
        'unit'        :      'psi',
        'reading'     :          0
    },

    /**
     * @description This operation initializes the runtime data and
     *              initiates the asyncronous loading of the gauge GUI.
     */
    _create: function()
    {
        var options = this.options;

        // normalize the scale of the gauge.
	this.scale_x = options.width / this._normalW;
        this.scale_y = options.height / this._normalH;

        // update the DOM element for this gauge.
        var self = $(this.element[0]);
        self.addClass('dial_gauge').css({
            // top : options.top + 'px;',
            // left: options.left + 'px',
            width: options.width + 'px',
            height: options.height + 'px'
        });

        // load in the gauge graphic.
        var handle = this;
        if (options.no_svg)
        {
            $.ajax({url: this._baseURL + 'no_svg/simple_reading_gauge.html',
                    dataType: 'text',
                    success: function(data, textStatus, jqXHR)
                    {
                        handle._ready(data, textStatus);
                    }
            });
        }
        else
        {
            self.svg({loadURL: this._baseURL + 'svg/simple_reading_gauge.svg',
                      onLoad: function(svg, error) {
                              handle._ready(svg, error);} });
        }
    },

    /**
     * @private
     * @description This operation applies any final operations to the DOM
     *              in preparation for runtime use. It expects the return
     *              signature produced by the SVG plug-in callback.
     * @param svg This argument provides access to the svg object.
     * @param error This argument, when defined, indicates an error.
     */
    _ready : function(svg, msg)
    {
        var options = this.options;
        if (typeof(msg) !== 'undefined')
        {
            if (msg === 'success')
            {
                // NOTE: no_svg === TRUE is assumed here.
                var self = $(this.element[0]);
                self.addClass('no_svg_' + options.skin);
                self.html(svg);


                this.ready = true;

                this._refresh();
            }
            else
            {
                //if (typeof(console.log) !== 'undefined')
                //{
                //    console.log(msg);
                //}
            }
        }
        else
        {
            var root = svg.root();
            this.svg = svg;

            if (options.skin === 'white')
            {
                $('rect.gaugeMask', root).each(function(index)
                    {
                        this.setAttribute('style', 'fill:url(#light_mat)');
                    }
                );
               $('.gaugeFont', root)[0].setAttribute('fill', '#0c0c0d');
            }

            this.ready = true;

            this._refresh();
        }
    },

    /**
     * @private
     * @description This operation takes no arguments. It will update all
     *              GUI elements of the SVG Gauge. It will have no effect
     *              if '_ready' is FALSE.
     */
    _refresh : function()
    {
        if (!this.ready)
            return;
        var options = this.options;
        var scale_x = this.scale_x;
        var scale_y = this.scale_y;
        var lowerScale = Math.min(scale_x, scale_y);

        if (options.no_svg)
        {
            var root = $(this.element[0]);
            $('.gaugeReading', root).text(options.reading).attr(
                'style', 'font-size:' +
                         (this.readingFont * lowerScale) + '%;');
            $('.gaugeUnit', root).text(options.unit).attr(
                'style', 'font-size:' +
                         (this.unitFont * lowerScale) + '%;');
            $('.gaugeLegend', root).text(options.legend).attr(
                'style', 'font-size:' +
                         (this.legendFont * lowerScale) + '%;');
        }
        else
        {
            var svg_root = this.svg.root();
            // update the rendering of the gauge.
            $('.gauge', svg_root).attr(
                'transform', 'scale(' + scale_x + ',' + scale_y + ')'
            );

            $('.gaugeReading', svg_root).text(options.reading).attr(
                {'x'         : this.readingX * scale_x,
                 'y'         : this.readingY * scale_y,
                 'font-size' : (this.readingFont * lowerScale) + '%'});

            $('.gaugeUnit', svg_root).text(options.unit).attr(
                {'x'         : this.unitX * scale_x,
                 'y'         : this.unitY * scale_y,
                 'font-size' : (this.unitFont * lowerScale) + '%'});

             $('.gaugeLegend', svg_root).text(options.legend).attr(
                {'x'         : this.legendX * scale_x,
                 'y'         : this.legendY * scale_y,
                 'font-size' : (this.legendFont * lowerScale) + '%'});
        }
    },

    /**
     * @description This operation updates the current configuration of
     *              the gauge and refreshes the display.
     * @param new_config This argument expects and object containing key
     *                   value pairs to be updated. Unspecified
     *                   configuration items will not be changed.
     */
    set_configuration : function( new_config )
    {
        if ( new_config ) {
            // Merge in the new configuration items.
            $.extend( this.options, new_config );
            
            var options = this.options;

            // normalize the scale of the gauge.
            this.scale_x = options.width / this._normalW;
            this.scale_y = options.height / this._normalH;

            // skip past render updates if not ready.
            if (this.ready && !options.no_svg)
            {
                // update the root to the new size.
                $('.gauge', this.svg.root()).parent().attr({
                    height: options.height,
                    width: options.width
                });
            }

            // redraw the gauge.
            this._refresh();
        }
    },

    /**
     * @description This operation updates the current reading of the
     *              gauge and updates the GUI when '_ready' is TRUE.
     * @param updated_reading This argument defines the new FLOAT value
     *                        that this gauge should reflect.
     */
    set_reading : function(updated_reading)
    {
        var options = this.options;
        var updated = parseFloat(updated_reading);

        // apply the value.
        options.reading = updated;
        if (this.ready)
        {
            if (!options.no_svg)
            {
                var svg_root = this.svg.root();
                $('.gaugeReading', svg_root).text(options.reading);
            }
            else
            {
                var root = $(this.element[0]);
                $('.gaugeReading', root).text(options.reading);
            }
        }
    },

    destroy: function() {
        var self = $(this.element[0]);
        self.svg('destroy');

        $.Widget.prototype.destroy.apply(this, arguments);
    }
});

}( jQuery ));

