(function($){
    "use strict";
    var plugin_name = 'armada_simulator';

    var sample_channels = [{
        name: 'seconds',
        description: 'Simulated GPS Decimal Seconds',
        label: 'Seconds',
        range_min: '0',
        range_max: '60',
        sample_rate: '5',
        units: 's',
        connector: 'seconds@gps-1.simulator'
    }, {
        name: 'minutes',
        description: 'Simulated GPS Minutes',
        label: 'Minutes',
        range_min: '0',
        range_max: '59',
        sample_rate: '5',
        units: 'minutes',
        connector: 'minutes@gps-1.simulator'
    }, {
        name: 'hours',
        description: 'Simulated GPS Hours',
        label: 'Hours',
        range_min: '0',
        range_max: '23',
        sample_rate: '5',
        units: 'hours',
        connector: 'hours@gps-1.simulator'
    }];

    var default_preamble = [
        ["#add_connector", "can-1.simulator"],
        ["#add_connector", "gps-1.simulator"],
        ["#add_channel", sample_channels]
    ];
    var genfn = {
        decimal_seconds: function(d){
            var secs = d.getUTCSeconds() + d.getUTCMilliseconds() * 0.001;
            return secs.toFixed(3);
        },
        minutes: function(d){
            return d.getUTCMinutes();
        },
        hours: function(d){
            return d.getUTCHours();
        }
    };

    var default_generators = {
        'seconds@gps-1.simulator': genfn.decimal_seconds,
        'minutes@gps-1.simulator': genfn.minutes,
        'hours@gps-1.simulator': genfn.hours
    };

    var methods = {
        init: function(options) {
            var settings = {
                dispatcher: null,
                preamble: default_preamble,
                live_generators: default_generators,
                rate: 5
            };
            return this.each(function(){
                var o = $.extend({}, settings, options);
                $(this).data(plugin_name, o);

                var generators = o.live_generators;
                var dispatcher = o.dispatcher;
                o.generate = function() {
                    var date = new Date();
                    var readings = {};
                    for (var name in generators) {
                        readings[name] = generators[name](date);
                    }
                    dispatcher('#live_data', readings);
                };
            });
        },
        start: function() {
            var o = this.data(plugin_name);

            // Run the preamble
            var dispatcher = o.dispatcher;
            var e = o.preamble;
            for (var i = 0; i < e.length; i++) {
                dispatcher(e[i][0], e[i][1]);
            }
            o.interval_id = window.setInterval(o.generate, 1000 / o.rate);
        },
        stop: function() {
            var o = this.data(plugin_name);
            window.clearInterval(o.interval_id);
            delete o.interval_id;
        }
    };

    $.fn[plugin_name] = function(method) {
        if (methods[method]) {
            return methods[method].apply(this,
                Array.prototype.slice.call(arguments, 1));
        } else if (typeof method === 'object' || !method) {
            return methods.init.apply(this, arguments);
        } else {
            $.error('jQuery.' + plugin_name + ': No such method ' + method);
        }
    };
}(jQuery));
